import { getNameOfServiceType } from "../helpers/getNameOfServiceType.js";
import { BaseServiceImplementationResolver, isServiceImplementationResolver, } from "./BaseServiceImplementationResolver.js";
import { ServiceRegistry } from "../di/ServiceRegistry.js";
import { FactoryServiceImplementationResolver } from "./FactoryServiceImplementationResolver.js";
export class RegistryServiceImplementationResolver extends BaseServiceImplementationResolver {
    implementation;
    get name() {
        return getNameOfServiceType(this.implementation);
    }
    constructor(implementation) {
        super();
        this.implementation = implementation;
    }
    *resolve(internalGetService, descriptor, requestedBy, requestedPath, mode) {
        let implementation = this.implementation;
        if (isServiceImplementationResolver(implementation)) {
            implementation = (yield* implementation.resolve(internalGetService, descriptor, requestedBy, requestedPath, mode));
        }
        const metadata = ServiceRegistry.get(implementation);
        if (metadata) {
            if (metadata.dependencies.length > 0) {
                const instanceDependencies = [];
                requestedPath = {
                    value: descriptor,
                    next: requestedPath,
                };
                for (const dependency of metadata.dependencies) {
                    instanceDependencies.push(yield* internalGetService(dependency, descriptor, requestedPath, mode));
                }
                return () => new implementation(...instanceDependencies);
            }
            return () => new implementation();
        }
        return yield* new FactoryServiceImplementationResolver(implementation).resolve(internalGetService, descriptor, requestedBy, requestedPath, mode);
    }
}
//# sourceMappingURL=RegistryServiceImplementationResolver.js.map